@@ -1,6 +1,6 @@
 /*
      This file is part of libextractor.
-     (C) 2002, 2003, 2004 Vidyut Samanta and Christian Grothoff
+     (C) 2002, 2003, 2004, 2005 Vidyut Samanta and Christian Grothoff
 
      libextractor is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -26,10 +26,13 @@
 #else
 #include <../../libltdl/ltdl.h>
 #endif
-#include <locale.h>
-#include <iconv.h>
-#ifndef MINGW
-#include <langinfo.h>
+
+#if HAVE_LIBBZ2
+#include <bzlib.h>
+#endif
+
+#if HAVE_ZLIB
+#include <zlib.h>
 #endif
 
 #define DEBUG 1
@@ -81,7 +84,7 @@ static const char *keywordTypes[] = {
   gettext_noop("magnification"),
   gettext_noop("release"),
   gettext_noop("group"),
-  gettext_noop("size"), 
+  gettext_noop("size"),
   gettext_noop("summary"),
   gettext_noop("packager"),
   gettext_noop("vendor"),
@@ -109,11 +112,30 @@ static const char *keywordTypes[] = {
   gettext_noop("media type"),
   gettext_noop("contact"),
   gettext_noop("binary thumbnail data"),
+  gettext_noop("publication date"),
+  gettext_noop("camera make"),
+  gettext_noop("camera model"),
+  gettext_noop("exposure"),
+  gettext_noop("aperture"),
+  gettext_noop("exposure bias"),
+  gettext_noop("flash"),
+  gettext_noop("flash bias"),
+  gettext_noop("focal length"),
+  gettext_noop("focal length (35mm equivalent)"),
+  gettext_noop("iso speed"),
+  gettext_noop("exposure mode"),
+  gettext_noop("metering mode"),
+  gettext_noop("macro mode"),
+  gettext_noop("image quality"),
+  gettext_noop("white balance"),
+  gettext_noop("orientation"),
+  gettext_noop("template"),
+  gettext_noop("split"),
   NULL,
 };
 
 /* the number of keyword types (for bounds-checking) */
-#define HIGHEST_TYPE_NUMBER 71
+#define HIGHEST_TYPE_NUMBER 90
 
 #ifdef HAVE_LIBOGG
 #if HAVE_VORBIS
@@ -125,6 +147,12 @@ static const char *keywordTypes[] = {
 #define WITH_OGG 1
 #endif
 
+#if HAVE_EXIV2
+#define EXSO "libextractor_exiv2:"
+#else
+#define EXSO ""
+#endif
+
 #if WITH_OGG
 #define OGGSO "libextractor_ogg:"
 #else
@@ -173,7 +201,7 @@ libextractor_elf:\
 libextractor_oo:\
 libextractor_asf"
 
-#define DEFAULT_LIBRARIES OLESO OGGSO QTSO DEFSO
+#define DEFAULT_LIBRARIES EXSO OLESO OGGSO QTSO DEFSO
 
 const char * EXTRACTOR_getDefaultLibraries() {
   return DEFAULT_LIBRARIES;
@@ -181,24 +209,16 @@ const char * EXTRACTOR_getDefaultLibraries() {
 
 /* ************library initialization ***************** */
 
-#ifdef MINGW
-void __attribute__ ((constructor)) le_win_init(void) {
-  InitWinEnv();
-}
-
-void __attribute__ ((destructor)) le_win_fini(void) {
-  ShutdownWinEnv();
-}
-#endif
-
 static char * old_dlsearchpath = NULL;
 
 /* using libtool, needs init! */
 void __attribute__ ((constructor)) le_ltdl_init(void) {
   int err;
 
+#if ENABLE_NLS
   setlocale(LC_ALL, "");
-  bindtextdomain(PACKAGE, LOCALEDIR);
+  BINDTEXTDOMAIN(PACKAGE, LOCALEDIR);
+#endif
   err = lt_dlinit ();
   if (err > 0)
     {
@@ -221,56 +241,65 @@ void __attribute__ ((constructor)) le_ltdl_init(void) {
   if (strstr (lt_dlgetsearchpath (), PLUGIN_PATH) == NULL)
     lt_dladdsearchdir (PLUGIN_PATH);
 #endif
+#ifdef MINGW
+  InitWinEnv();
+#endif
 }
 
 void __attribute__ ((destructor)) le_ltdl_fini(void) {
   lt_dlsetsearchpath(old_dlsearchpath);
-  if (old_dlsearchpath != NULL)
+  if (old_dlsearchpath != NULL) {
     free(old_dlsearchpath);
+    old_dlsearchpath = NULL;
+  }
+#ifdef MINGW
+  ShutdownWinEnv();
+#endif
+
   lt_dlexit ();
 }
 
 /**
  * Open a file
  */
-static int OPEN(const char *filename, 
-		int oflag, ...) {
+static int fileopen(const char *filename, int oflag, ...)
+{
   int mode;
   char *fn;
 
 #ifdef MINGW
   char szFile[_MAX_PATH + 1];
   long lRet;
-  
-  if ((lRet = conv_to_win_path(filename, szFile)) != ERROR_SUCCESS)
+
+  if ((lRet = plibc_conv_to_win_path(filename, szFile)) != ERROR_SUCCESS)
   {
     errno = ENOENT;
     SetLastError(lRet);
-    
+
     return -1;
   }
   fn = szFile;
 #else
   fn = (char *) filename;
 #endif
-  
+
   if (oflag & O_CREAT)
   {
     va_list arg;
     va_start(arg, oflag);
     mode = va_arg(arg, int);
-    va_end(arg);    
+    va_end(arg);
   }
   else
   {
     mode = 0;
   }
-  
+
 #ifdef MINGW
   /* Set binary mode */
   mode |= O_BINARY;
 #endif
-  
+
   return open(fn, oflag, mode);
 }
 
@@ -278,7 +307,7 @@ static int OPEN(const char *filename,
 
 /**
  * Load the default set of libraries. The default set of
- * libraries consists of the libraries that are part of 
+ * libraries consists of the libraries that are part of
  * the libextractor distribution (except split and filename
  * extractor) plus the extractors that are specified
  * in the environment variable "LIBEXTRACTOR_LIBRARIES".
@@ -320,52 +349,6 @@ EXTRACTOR_getKeywordTypeAsString(const EXTRACTOR_KeywordType type)
     return NULL;
 }
 
-/**
- * Load multiple libraries as specified by the user.
- * @param config a string given by the user that defines which
- *        libraries should be loaded. Has the format
- *        "[[-]LIBRARYNAME[:[-]LIBRARYNAME]*]". For example,
- *        libextractor_mp3.so:libextractor_ogg.so loads the
- *        mp3 and the ogg library. The '-' before the LIBRARYNAME
- *        indicates that the library should be added to the end
- *        of the library list (addLibraryLast).
- * @param prev the  previous list of libraries, may be NULL
- * @return the new list of libraries, equal to prev iff an error occured
- *         or if config was empty (or NULL).
- */
-EXTRACTOR_ExtractorList *
-EXTRACTOR_loadConfigLibraries (EXTRACTOR_ExtractorList * prev,
-			       const char *config)
-{
-  char *cpy;
-  int pos;
-  int last;
-  int len;
-
-  if (config == NULL)
-    return prev;
-  len = strlen(config);
-  cpy = strdup(config);
-  pos = 0;
-  last = 0;
-  while (pos < len)
-    {
-      while ((cpy[pos] != ':') && (cpy[pos] != '\0'))
-	pos++;
-      cpy[pos++] = '\0';	/* replace ':' by termination */
-      if (cpy[last] == '-')
-	{
-	  last++;
-	  prev = EXTRACTOR_addLibraryLast (prev, &cpy[last]);
-	}
-      else
-	prev = EXTRACTOR_addLibrary (prev, &cpy[last]);
-      last = pos;
-    }
-  free (cpy);
-  return prev;
-}
-
 static void *getSymbolWithPrefix(void *lib_handle,
                                  const char *lib_name,
                                  const char *sym_name)
@@ -390,9 +373,9 @@ static void *getSymbolWithPrefix(void *lib_handle,
     symbol=lt_dlsym(lib_handle,name /* now try with the '_' */);
 #if DEBUG
     fprintf(stderr,
-	    _("Resolving symbol '%s' in library '%s' failed, "
-	      "so I tried '%s', but that failed also.  Errors are: "
-	      "'%s' and '%s'.\n"),
+	    _("Resolving symbol `%s' in library `%s' failed, "
+	      "so I tried `%s', but that failed also.  Errors are: "
+	      "`%s' and `%s'.\n"),
              name+1,
              lib_name,
              name,
@@ -411,7 +394,7 @@ static void *getSymbolWithPrefix(void *lib_handle,
  */
 static int
 loadLibrary (const char *name,
-	     void **libHandle, 
+	     void **libHandle,
 	     ExtractMethod * method)
 {
   *libHandle = lt_dlopenext (name);
@@ -419,8 +402,8 @@ loadLibrary (const char *name,
     {
 #if DEBUG
       fprintf (stderr,
-	       _("Loading '%s' plugin failed: %s\n"), 
-	       name, 
+	       _("Loading `%s' plugin failed: %s\n"),
+	       name,
 	       lt_dlerror ());
 #endif
       return -1;
@@ -434,15 +417,10 @@ loadLibrary (const char *name,
   return 1;
 }
 
-/**
- * Add a library for keyword extraction.
- * @param prev the previous list of libraries, may be NULL
- * @param library the name of the library
- * @return the new list of libraries, equal to prev iff an error occured
- */
-EXTRACTOR_ExtractorList *
-EXTRACTOR_addLibrary (EXTRACTOR_ExtractorList * prev, 
-		      const char *library)
+/* Internal function that accepts options. */
+static EXTRACTOR_ExtractorList *
+EXTRACTOR_addLibrary2 (EXTRACTOR_ExtractorList * prev,
+		       const char *library, const char *options)
 {
   EXTRACTOR_ExtractorList *result;
   void *handle;
@@ -455,19 +433,30 @@ EXTRACTOR_addLibrary (EXTRACTOR_ExtractorList * prev,
   result->libraryHandle = handle;
   result->extractMethod = method;
   result->libname = strdup (library);
+  if( options )
+    result->options = strdup (options);
+  else
+    result->options = NULL;
   return result;
 }
 
 /**
- * Add a library for keyword extraction at the END of the list.
+ * Add a library for keyword extraction.
  * @param prev the previous list of libraries, may be NULL
  * @param library the name of the library
- * @return the new list of libraries, always equal to prev
- *         except if prev was NULL and no error occurs
+ * @return the new list of libraries, equal to prev iff an error occured
  */
 EXTRACTOR_ExtractorList *
-EXTRACTOR_addLibraryLast(EXTRACTOR_ExtractorList * prev, 
-			 const char *library)
+EXTRACTOR_addLibrary (EXTRACTOR_ExtractorList * prev,
+		      const char *library)
+{
+  return EXTRACTOR_addLibrary2(prev, library, NULL);
+}
+
+/* Internal function which takes options. */
+static EXTRACTOR_ExtractorList *
+EXTRACTOR_addLibraryLast2 (EXTRACTOR_ExtractorList * prev,
+			   const char *library, const char *options)
 {
   EXTRACTOR_ExtractorList *result;
   EXTRACTOR_ExtractorList *pos;
@@ -481,6 +470,10 @@ EXTRACTOR_addLibraryLast(EXTRACTOR_ExtractorList * prev,
   result->libraryHandle = handle;
   result->extractMethod = method;
   result->libname = strdup (library);
+  if( options )
+    result->options = strdup (options);
+  else
+    result->options = NULL;
   if (prev == NULL)
     return result;
   pos = prev;
@@ -490,6 +483,93 @@ EXTRACTOR_addLibraryLast(EXTRACTOR_ExtractorList * prev,
   return prev;
 }
 
+/**
+ * Add a library for keyword extraction at the END of the list.
+ * @param prev the previous list of libraries, may be NULL
+ * @param library the name of the library
+ * @return the new list of libraries, always equal to prev
+ *         except if prev was NULL and no error occurs
+ */
+EXTRACTOR_ExtractorList *
+EXTRACTOR_addLibraryLast (EXTRACTOR_ExtractorList * prev,
+			  const char *library)
+{
+  return EXTRACTOR_addLibraryLast2(prev, library, NULL);
+}
+
+/**
+ * Load multiple libraries as specified by the user.
+ * @param config a string given by the user that defines which
+ *        libraries should be loaded. Has the format
+ *        "[[-]LIBRARYNAME[:[-]LIBRARYNAME]*]". For example,
+ *        libextractor_mp3.so:libextractor_ogg.so loads the
+ *        mp3 and the ogg library. The '-' before the LIBRARYNAME
+ *        indicates that the library should be added to the end
+ *        of the library list (addLibraryLast).
+ * @param prev the  previous list of libraries, may be NULL
+ * @return the new list of libraries, equal to prev iff an error occured
+ *         or if config was empty (or NULL).
+ */
+EXTRACTOR_ExtractorList *
+EXTRACTOR_loadConfigLibraries (EXTRACTOR_ExtractorList * prev,
+			       const char *config)
+{
+  char *cpy;
+  int pos;
+  int last;
+  int lastconf;
+  int len;
+
+  if (config == NULL)
+    return prev;
+  len = strlen(config);
+  cpy = strdup(config);
+  pos = 0;
+  last = 0;
+  lastconf = 0;
+  while (pos < len)
+    {
+      while ((cpy[pos] != ':') && (cpy[pos] != '\0') &&
+	     (cpy[pos] != '('))
+	pos++;
+      if( cpy[pos] == '(' ) {
+ 	cpy[pos++] = '\0';	/* replace '(' by termination */
+	lastconf = pos;         /* start config from here, after (. */
+	while ((cpy[pos] != '\0') && (cpy[pos] != ')'))
+	  pos++; /* config until ) or EOS. */
+	if( cpy[pos] == ')' ) {
+	  cpy[pos++] = '\0'; /* write end of config here. */
+	  while ((cpy[pos] != ':') && (cpy[pos] != '\0'))
+	    pos++; /* forward until real end of string found. */
+	  cpy[pos++] = '\0';
+	} else {
+	  cpy[pos++] = '\0'; /* end of string. */
+	}
+      } else {
+	lastconf = -1;         /* NULL config when no (). */
+	cpy[pos++] = '\0';	/* replace ':' by termination */
+      }
+      if (cpy[last] == '-')
+	{
+	  last++;
+	  if( lastconf != -1 )
+	    prev = EXTRACTOR_addLibraryLast2 (prev, &cpy[last],
+					      &cpy[lastconf]);
+	  else
+	    prev = EXTRACTOR_addLibraryLast2 (prev, &cpy[last], NULL);
+	}
+      else
+	if( lastconf != -1 )
+	  prev = EXTRACTOR_addLibrary2 (prev, &cpy[last], &cpy[lastconf]);
+	else
+	  prev = EXTRACTOR_addLibrary2 (prev, &cpy[last], NULL);
+
+      last = pos;
+    }
+  free (cpy);
+  return prev;
+}
+
 /**
  * Remove a library for keyword extraction.
  * @param prev the current list of libraries
@@ -497,7 +577,7 @@ EXTRACTOR_addLibraryLast(EXTRACTOR_ExtractorList * prev,
  * @return the reduced list, unchanged if the library was not loaded
  */
 EXTRACTOR_ExtractorList *
-EXTRACTOR_removeLibrary(EXTRACTOR_ExtractorList * prev, 
+EXTRACTOR_removeLibrary(EXTRACTOR_ExtractorList * prev,
 			const char *library)
 {
   EXTRACTOR_ExtractorList *pos;
@@ -518,13 +598,16 @@ EXTRACTOR_removeLibrary(EXTRACTOR_ExtractorList * prev,
 	prev->next = pos->next;
       /* found */
       free (pos->libname);
-      lt_dlclose (pos->libraryHandle);
+      if( pos->options )
+	free (pos->options);
+      if( pos->libraryHandle )
+	lt_dlclose (pos->libraryHandle);
       free (pos);
     }
 #if DEBUG
   else
-    fprintf(stderr, 
-	    _("Unloading plugin '%s' failed!\n"), 
+    fprintf(stderr,
+	    _("Unloading plugin `%s' failed!\n"),
 	    library);
 #endif
   return first;
@@ -541,6 +624,253 @@ EXTRACTOR_removeAll (EXTRACTOR_ExtractorList * libraries)
     libraries = EXTRACTOR_removeLibrary (libraries, libraries->libname);
 }
 
+
+
+/**
+ * How many bytes do we actually try to scan? (from the beginning
+ * of the file).  Limit to 1 GB.
+ */
+#define MAX_READ 1024 * 1024 * 1024
+
+/**
+ * How many bytes do we actually try to decompress? (from the beginning
+ * of the file).  Limit to 16 MB.
+ */
+#define MAX_DECOMPRESS 16 * 1024 * 1024
+
+
+static EXTRACTOR_KeywordList *
+getKeywords (EXTRACTOR_ExtractorList * extractor,
+	     const char * filename,
+	     const unsigned char * data,
+	     size_t size) {
+  EXTRACTOR_KeywordList *result;
+  unsigned char * buf;
+  size_t dsize;
+#if HAVE_ZLIB
+  z_stream strm;
+  int ret;
+  size_t pos;
+#endif
+#if HAVE_LIBBZ2
+  bz_stream bstrm;
+  int bret;
+  size_t bpos;
+#endif
+
+  result = NULL;
+  buf = NULL;
+  dsize = 0;
+#if HAVE_ZLIB
+  /* try gzip decompression first */
+  if ( (size >= 12) &&
+       (data[0] == 0x1f) &&
+       (data[1] == 0x8b) &&
+       (data[2] == 0x08) ) {
+
+    /*
+     * Skip gzip header - we might want to retrieve parts of it as keywords
+     */
+    unsigned gzip_header_length = 10;
+
+    if (data[3] & 0x4) /* FEXTRA  set */
+      gzip_header_length += 2 + (unsigned) (data[10] & 0xff)
+                              + (((unsigned) (data[11] & 0xff)) * 256);
+
+    if(data[3] & 0x8) /* FNAME set */
+    {
+      const unsigned char * cptr = data + gzip_header_length;
+
+      /*
+       * stored file name is here
+       * extremely long file names might break the following code.
+       */
+
+      while(cptr < data + size)
+      {
+        if('\0' == *cptr)
+          break;
+
+        cptr++;
+      }
+      gzip_header_length = (cptr - data) + 1;
+    }
+
+    if(data[3] & 0x16) /* FCOMMENT set */
+    {
+      const unsigned char * cptr = data + gzip_header_length;
+
+      /*
+       * stored comment is here
+       */
+
+      while(cptr < data + size)
+      {
+        if('\0' == *cptr)
+          break;
+
+        cptr ++;
+      }
+
+      gzip_header_length = (cptr - data) + 1;
+    }
+
+    if(data[3] & 0x2) /* FCHRC set */
+      gzip_header_length += 2;
+
+    memset(&strm,
+	   0,
+	   sizeof(z_stream));
+#ifdef ZLIB_VERNUM 
+    gzip_header_length = 0;
+#endif
+    if (size > gzip_header_length) {
+      strm.next_in = (Bytef*) data + gzip_header_length;
+      strm.avail_in = size - gzip_header_length;
+    } else {
+      strm.next_in = (Bytef*) data;
+      strm.avail_in = 0;
+    }
+    strm.total_in = 0;
+    strm.zalloc = NULL;
+    strm.zfree = NULL;
+    strm.opaque = NULL;
+    
+    /*
+     * note: maybe plain inflateInit(&strm) is adequate,
+     * it looks more backward-compatible also ;
+     *
+     * ZLIB_VERNUM isn't defined by zlib version 1.1.4 ;
+     * there might be a better check.
+     */ 
+#ifdef ZLIB_VERNUM
+    if (Z_OK == inflateInit2(&strm,
+			     15 + 32)) {
+#else
+    if (Z_OK == inflateInit2(&strm,
+			     -MAX_WBITS)) {
+#endif
+      dsize = 2 * size;
+      if (dsize > MAX_DECOMPRESS)
+	dsize = MAX_DECOMPRESS;
+      buf = malloc(dsize);
+      pos = 0;
+      if (buf == NULL) {
+	inflateEnd(&strm);
+      } else {
+	strm.next_out = (Bytef*) buf;
+	strm.avail_out = dsize;
+	do {
+	  ret = inflate(&strm,
+			Z_SYNC_FLUSH);
+	  if (ret == Z_OK) {
+	    if (dsize == MAX_DECOMPRESS)
+	      break;
+	    pos += strm.total_out;
+	    strm.total_out = 0;
+	    dsize *= 2;
+	    if (dsize > MAX_DECOMPRESS)
+	      dsize = MAX_DECOMPRESS;
+	    buf = realloc(buf, dsize);
+	    strm.next_out = (Bytef*) &buf[pos];
+	    strm.avail_out = dsize - pos;
+	  } else if (ret != Z_STREAM_END) {
+	    /* error */
+	    free(buf);
+	    buf = NULL;
+	  }
+	} while ( (buf != NULL) &&		  
+		  (ret != Z_STREAM_END) );      
+	dsize = pos + strm.total_out; 
+	inflateEnd(&strm);
+	if (dsize == 0) {
+	  free(buf);
+	  buf = NULL;
+	}
+      }
+    }
+  }
+#endif
+
+#if HAVE_LIBBZ2
+  if ( (data[0] == 'B') &&
+       (data[1] == 'Z') &&
+       (data[2] == 'h') ) {
+    /* now try bz2 decompression */
+    memset(&bstrm,
+	   0,
+	   sizeof(bz_stream));
+    bstrm.next_in = (char*) data;
+    bstrm.avail_in = size;
+    bstrm.total_in_lo32 = 0;
+    bstrm.total_in_hi32 = 0;
+    bstrm.bzalloc = NULL;
+    bstrm.bzfree = NULL;
+    bstrm.opaque = NULL;
+    if ( (buf == NULL) &&
+	 (BZ_OK == BZ2_bzDecompressInit(&bstrm,
+					0,
+					0)) ) {
+      dsize = 2 * size;
+      if (dsize > MAX_DECOMPRESS)
+	dsize = MAX_DECOMPRESS;
+      buf = malloc(dsize);
+      bpos = 0;
+      if (buf == NULL) {
+	BZ2_bzDecompressEnd(&bstrm);
+      } else {
+	bstrm.next_out = (char*) buf;
+	bstrm.avail_out = dsize;
+	do {
+	  bret = BZ2_bzDecompress(&bstrm);
+	  if (bret == Z_OK) {
+	    if (dsize == MAX_DECOMPRESS) 
+	      break;
+	    bpos += bstrm.total_out_lo32;
+	    bstrm.total_out_lo32 = 0;
+	    dsize *= 2;
+	    if (dsize > MAX_DECOMPRESS)
+	      dsize = MAX_DECOMPRESS;
+	    buf = realloc(buf, dsize);
+	    bstrm.next_out = (char*) &buf[bpos];
+	    bstrm.avail_out = dsize - bpos;
+	  } else if (bret != BZ_STREAM_END) {
+	    /* error */
+	    free(buf);
+	    buf = NULL;
+	  }
+	} while ( (buf != NULL) &&
+		  (bret != BZ_STREAM_END) );      
+	dsize = bpos + bstrm.total_out_lo32; 
+	BZ2_bzDecompressEnd(&bstrm);
+	if (dsize == 0) {
+	  free(buf);
+	  buf = NULL;
+	}
+      }
+    }
+  }
+#endif
+
+
+  /* finally, call plugins */
+  if (buf != NULL) {
+    data = buf;
+    size = dsize;
+  }
+  while (extractor != NULL) {
+    result = extractor->extractMethod(filename,
+				      (char*) data,
+				      size,
+				      result,
+				      extractor->options);
+    extractor = extractor->next;
+  }
+  if (buf != NULL)
+    free(buf);
+  return result;
+}
+
 /**
  * Extract keywords from a file using the available extractors.
  * @param extractor the list of extractor libraries
@@ -550,17 +880,18 @@ EXTRACTOR_removeAll (EXTRACTOR_ExtractorList * libraries)
  */
 EXTRACTOR_KeywordList *
 EXTRACTOR_getKeywords (EXTRACTOR_ExtractorList * extractor,
-		       const char *filename) {
+		       const char * filename) {
   EXTRACTOR_KeywordList *result;
   int file;
-  char * buffer;
+  void * buffer;
   struct stat fstatbuf;
   size_t size;
+  int eno;
 
 #ifdef O_LARGEFILE
-  file = OPEN(filename, O_RDONLY | O_LARGEFILE);
+  file = fileopen(filename, O_RDONLY | O_LARGEFILE);
 #else
-  file = OPEN(filename, O_RDONLY);
+  file = fileopen(filename, O_RDONLY);
 #endif
   if (-1 == file)
     return NULL;
@@ -574,46 +905,53 @@ EXTRACTOR_getKeywords (EXTRACTOR_ExtractorList * extractor,
     return NULL;
   }
 
-#ifndef MINGW
-  if (size > 1* 1024 * 1024 * 1024)
-    size = 1 * 1024 * 1024 * 1024; /* do not mmap/read more than 1 GB! */
-  buffer = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_PRIVATE, file, 0);
-#else
-  if (size > 250 * 1024 * 1024)
-    size = 250 * 1024 * 1024; /* do not malloc/read more than 250 MB! */
-  buffer = malloc(size + 1);
-  if (buffer == NULL) { /* out of memory? */
+  if (size > MAX_READ)
+    size = MAX_READ; /* do not mmap/read more than 1 GB! */
+  buffer = MMAP(NULL, size, PROT_READ, MAP_PRIVATE, file, 0);
+  if ( (buffer == NULL) || (buffer == (void *) -1) ) {
+    eno = errno;
     close(file);
+    errno = eno;
     return NULL;
   }
-  buffer[size] = 0;
-  if (size > 0)
-    read(file, buffer, size);
-#endif
+  result = getKeywords(extractor,
+		       filename,
+		       buffer,
+		       size);
+  MUNMAP (buffer, size);
   close(file);
-  if ( (buffer == NULL) || (buffer == (void *) -1) )
-    return NULL;
-  result = NULL;
-  while (extractor != NULL) {
-    result = extractor->extractMethod (filename, buffer, size, result);
-    extractor = extractor->next;
-  }
-#ifndef MINGW
-  if (size > 0)
-    munmap (buffer, size);
-  else
-    free(buffer);
-#else
-  free(buffer);
-#endif
   return result;
 }
 
+
+
+/**
+ * Extract keywords from a buffer in memory
+ * using the available extractors.
+ *
+ * @param extractor the list of extractor libraries
+ * @param data the data of the file
+ * @param size the number of bytes in data
+ * @return the list of keywords found in the file, NULL if none
+ *         were found (or other errors)
+ */
+EXTRACTOR_KeywordList *
+EXTRACTOR_getKeywords2(EXTRACTOR_ExtractorList * extractor,
+		       const void * data,
+		       size_t size) {
+  if (data == NULL)
+    return NULL;
+  return getKeywords(extractor,
+		     NULL,
+		     data,
+		     size);
+}
+
 static void
 removeKeyword (const char *keyword,
 	       const EXTRACTOR_KeywordType type,
 	       const unsigned int options,
-	       EXTRACTOR_KeywordList ** list, 
+	       EXTRACTOR_KeywordList ** list,
 	       EXTRACTOR_KeywordList * current)
 {
   EXTRACTOR_KeywordList *first;
@@ -700,7 +1038,7 @@ EXTRACTOR_removeEmptyKeywords (EXTRACTOR_KeywordList * list)
       for (i=strlen(pos->keyword)-1;i>=0;i--)
 	if (! isspace(pos->keyword[i]))
 	  allWhite = 0;
-      if (allWhite) 
+      if (allWhite)
 	{
 	  EXTRACTOR_KeywordList * next;
 	  next = pos->next;
@@ -712,7 +1050,7 @@ EXTRACTOR_removeEmptyKeywords (EXTRACTOR_KeywordList * list)
 	  free(pos);
 	  pos = next;
 	}
-      else 
+      else
 	{
 	  last = pos;
 	  pos = pos->next;
@@ -721,6 +1059,39 @@ EXTRACTOR_removeEmptyKeywords (EXTRACTOR_KeywordList * list)
   return list;
 }
 
+/**
+ * Remove keywords of a particular type from the list.
+ * @param list the original keyword list (altered in the process!)
+ * @param type the type to remove
+ * @return a list of keywords without entries of given type
+ */
+EXTRACTOR_KeywordList *
+EXTRACTOR_removeKeywordsOfType(EXTRACTOR_KeywordList * list,
+			       EXTRACTOR_KeywordType type) {
+  EXTRACTOR_KeywordList * pos;
+  EXTRACTOR_KeywordList * last;
+
+  last = NULL;
+  pos = list;
+  while (pos != NULL) {
+    if (pos->keywordType == type) {
+      EXTRACTOR_KeywordList * next;
+      next = pos->next;
+      if (last == NULL)
+	list = next;
+      else
+	last->next = next;
+      free(pos->keyword);
+      free(pos);
+      pos = next;
+    } else {
+      last = pos;
+      pos = pos->next;
+    }
+  }
+  return list;
+}
+
 #include "iconv.c"
 
 /**
@@ -730,18 +1101,14 @@ EXTRACTOR_removeEmptyKeywords (EXTRACTOR_KeywordList * list)
  * @param keywords the list of keywords to print, may be NULL
  */
 void
-EXTRACTOR_printKeywords(FILE * handle, 
+EXTRACTOR_printKeywords(FILE * handle,
 			EXTRACTOR_KeywordList * keywords)
 {
   iconv_t cd;
   char * buf;
 
   cd = iconv_open(
-#ifdef MINGW
-    ""
-#else
     nl_langinfo(CODESET)
-#endif
     , "UTF-8");
   while (keywords != NULL)
     {
@@ -754,9 +1121,9 @@ EXTRACTOR_printKeywords(FILE * handle,
 	fprintf(handle,
 		_("%s - (binary)\n"),
 		gettext(keywordTypes[keywords->keywordType]));
-      } else {   
+      } else {
 	if (keywords->keywordType >= HIGHEST_TYPE_NUMBER)
-	  fprintf(handle, 
+	  fprintf(handle,
 		  _("INVALID TYPE - %s\n"),
 		  buf);
 	else
@@ -826,7 +1193,7 @@ EXTRACTOR_extractLast (const EXTRACTOR_KeywordType type,
  * @return the last matching keyword, or NULL if none matches
  */
 const char *
-EXTRACTOR_extractLastByString (const char * type, 
+EXTRACTOR_extractLastByString (const char * type,
 			       EXTRACTOR_KeywordList * keywords)
 {
   char * result = NULL;
@@ -858,209 +1225,109 @@ EXTRACTOR_countKeywords (EXTRACTOR_KeywordList * keywords)
   return count;
 }
 
-
-
-/* ******************* Java support *********************** */
-
-#ifdef HAVE_JNI_H
-
-#include <jni.h>
-
-/* gcj's jni.h does not define JNIEXPORT/JNICALL (at least
- * not in my version).  Sun defines it to 'empty' in Linux,
- * so that should work */
-#ifndef JNIEXPORT
-#define JNIEXPORT
-#endif
-#ifndef JNICALL
-#define JNICALL
-#endif
-
-#include "org_ovmj_libextractor_Extractor.h"
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    loadDefaultInternal
- * Signature: ()J
+/**
+ * Encode the given binary data object
+ * as a 0-terminated C-string according
+ * to the LE binary data encoding standard.
+ *
+ * @return NULL on error, the 0-terminated
+ *  encoding otherwise
  */
-JNIEXPORT jlong JNICALL Java_org_ovmj_libextractor_Extractor_loadDefaultInternal(JNIEnv * env,
-										 jclass c) {
-  return (jlong) (long) EXTRACTOR_loadDefaultLibraries();
-}
+char * EXTRACTOR_binaryEncode(const unsigned char * data,
+			      size_t size) {
 
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    unloadInternal
- * Signature: (J)V
- */
-JNIEXPORT void JNICALL Java_org_ovmj_libextractor_Extractor_unloadInternal(JNIEnv * env, 
-									   jclass c,
-									   jlong arg) {
-  EXTRACTOR_removeAll((EXTRACTOR_ExtractorList*) (long) arg);
-}
+  char * binary;
+  size_t pos;
+  size_t end;
+  size_t wpos;
+  size_t i;
+  unsigned int markers[8]; /* 256 bits */
+  unsigned char marker;
 
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    extractInternal
- * Signature: (JLjava/lang/String;)J
- */
-JNIEXPORT jlong JNICALL Java_org_ovmj_libextractor_Extractor_extractInternal(JNIEnv * env, 
-									     jclass c,
-									     jlong arg, 
-									     jstring f) {
-  const char * fname;
-  jboolean bo;
-  jlong ret;
-
-  bo = JNI_FALSE;
-  fname = (*env)->GetStringUTFChars(env, f, &bo);
-  ret = (jlong) (long) EXTRACTOR_getKeywords((EXTRACTOR_ExtractorList*) (long) arg,
-					     fname);
-  (*env)->ReleaseStringUTFChars(env, f, fname);
-  return ret;
-}
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    freeInternal
- * Signature: (J)V
- */
-JNIEXPORT void JNICALL Java_org_ovmj_libextractor_Extractor_freeInternal(JNIEnv * env, 
-									 jclass c, 
-									 jlong list) {
-  EXTRACTOR_freeKeywords( (EXTRACTOR_KeywordList*) (long) list);
-}
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    typeInternal
- * Signature: (J)I
- */
-JNIEXPORT jint JNICALL Java_org_ovmj_libextractor_Extractor_typeInternal(JNIEnv * env, 
-									 jclass c, 
-									 jlong list) {
-  if (list == 0)
-    return -1; /* error! */
-  return ((EXTRACTOR_KeywordList*) (long) list)->keywordType;
-}
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    keywordInternal
- * Signature: (J)Ljava/lang/String;
- */
-JNIEXPORT jstring JNICALL Java_org_ovmj_libextractor_Extractor_keywordInternal(JNIEnv * env, 
-									       jclass c, 
-									       jlong list) {
-  char * str;
-  if (list == 0)
-    return NULL; /* error! */
-  str = ((EXTRACTOR_KeywordList*) (long) list)->keyword;
-  return (*env)->NewStringUTF(env, str);
-}
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    nextInternal
- * Signature: (J)J
- */
-JNIEXPORT jlong JNICALL Java_org_ovmj_libextractor_Extractor_nextInternal(JNIEnv * env, 
-									  jclass j, 
-									  jlong list) {
-  if (list == 0)
-    return 0; /* error! */
-  return (jlong) (long) ((EXTRACTOR_KeywordList*) (long) list)->next;
-}
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    getTypeAsStringInternal
- * Signature: (I)Ljava/lang/String;
- */
-JNIEXPORT jstring JNICALL Java_org_ovmj_libextractor_Extractor_getTypeAsStringInternal(JNIEnv * env, 
-										       jclass c, 
-										       jint type) {
-  const char * str;
-  if ( (type < 0) || (type > HIGHEST_TYPE_NUMBER) )
-    return NULL; /* error! */  
-  str = gettext(keywordTypes[type]);
-  if (str == NULL)
+ /* encode! */
+  binary = malloc(2 + size + (size+256) / 254);
+  if (binary == NULL)
     return NULL;
-  return (*env)->NewStringUTF(env, 
-			      str);
-}
 
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    getVersionInternal
- * Signature: ()I
- */
-JNIEXPORT jint JNICALL Java_org_ovmj_libextractor_Extractor_getVersionInternal(JNIEnv * env, 
-									       jclass c) {
-  return EXTRACTOR_VERSION;
-}
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    getMaxTypeInternal
- * Signature: ()I
- */
-JNIEXPORT jint JNICALL Java_org_ovmj_libextractor_Extractor_getMaxTypeInternal(JNIEnv * env, 
-									       jclass c) {
-  return HIGHEST_TYPE_NUMBER;
-}
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    unloadPlugin
- * Signature: (JLjava/lang/String;)J
- */
-JNIEXPORT jlong JNICALL Java_org_ovmj_libextractor_Extractor_unloadPlugin(JNIEnv * env, 
-									  jclass c, 
-									  jlong handle, 
-									  jstring name) {
-  const char * lname;
-  jboolean bo;
-  jlong ret;
-
-  bo = JNI_FALSE;
-  lname = (*env)->GetStringUTFChars(env, name, &bo);
-  ret = (jlong) (long) EXTRACTOR_removeLibrary((EXTRACTOR_ExtractorList*) (long) handle,
-					       lname);
-  (*env)->ReleaseStringUTFChars(env, name, lname);
-  return ret; 
-}
-
-/*
- * Class:     org_ovmj_libextractor_Extractor
- * Method:    loadPlugin
- * Signature: (JLjava/lang/String;Z)J
- */
-JNIEXPORT jlong JNICALL Java_org_ovmj_libextractor_Extractor_loadPlugin(JNIEnv * env, 
-									jclass c, 
-									jlong handle, 
-									jstring name,
-									jboolean place) {
-  const char * lname;
-  jboolean bo;
-  jlong ret;
-
-  bo = JNI_FALSE;
-  lname = (*env)->GetStringUTFChars(env, name, &bo);
-  if (place == JNI_TRUE) {
-    ret = (jlong) (long) EXTRACTOR_addLibraryLast((EXTRACTOR_ExtractorList*) (long) handle,
-						  lname);
-  } else {
-    ret = (jlong) (long) EXTRACTOR_addLibrary((EXTRACTOR_ExtractorList*) (long) handle,
-					      lname);    
+  pos = 0;
+  wpos = 0;
+  while (pos < size) {
+    /* find unused value between 1 and 255 in
+       the next 254 bytes */
+    end = pos + 254;
+    if (end < pos)
+      break; /* integer overflow! */
+    if (end > size)
+      end = size;
+    memset(markers,
+	   0,
+	   sizeof(markers));
+    for (i=pos;i<end;i++)
+      markers[data[i]&7] |= 1 << (data[i] >> 3);
+    marker = 1;
+    while (markers[marker&7] & (1 << (marker >> 3))) {
+      marker++;
+      if (marker == 0) {
+	/* assertion failed... */
+	free(binary);
+	return NULL;
+      }
+    }
+    /* recode */
+    binary[wpos++] = marker;
+    for (i=pos;i<end;i++)
+      binary[wpos++] = data[i] == 0 ? marker : data[i];
+    pos = end;
   }
-  (*env)->ReleaseStringUTFChars(env, name, lname);
-  return ret; 
+  binary[wpos++] = 0; /* 0-termination! */
+  return binary;
+}
+
+
+/**
+ * This function can be used to decode the binary data
+ * encoded in the libextractor metadata (i.e. for
+ * the  thumbnails).
+ *
+ * @param in 0-terminated string from the meta-data
+ * @return 1 on error, 0 on success
+ */
+int EXTRACTOR_binaryDecode(const char * in,
+			   unsigned char ** out,
+			   size_t * outSize) {
+  unsigned char * buf;
+  size_t pos;
+  size_t wpos;
+  unsigned char marker;
+  size_t i;
+  size_t end;
+  size_t inSize;
+
+  inSize = strlen(in);
+  if (inSize == 0) {
+    *out = NULL;
+    *outSize = 0;
+    return 0;
+  }
+
+  buf = malloc(inSize); /* slightly more than needed ;-) */
+  *out = buf;
+
+  pos = 0;
+  wpos = 0;
+  while (pos < inSize) {
+    end = pos + 255; /* 255 here: count the marker! */
+    if (end > inSize)
+      end = inSize;
+    marker = in[pos++];
+    for (i=pos;i<end;i++)
+      buf[wpos++] = (in[i] == marker) ? 0 : in[i];
+    pos = end;
+  }
+  *outSize = wpos;
+  return 0;
 }
 
-/* #ifdef HAVE_JNI_H */
-#endif
 
 
 /* end of extractor.c */
