@@ -21,7 +21,6 @@
 #include "platform.h"
 #include "extractor.h"
 #include <zlib.h>
-#include <pthread.h>
 
 /*
  * The .deb is an ar-chive file.  It contains a tar.gz file
@@ -43,7 +42,7 @@ static EXTRACTOR_KeywordList * addKeyword(EXTRACTOR_KeywordType type,
   if (keyword == NULL)
     return next;
   result = malloc(sizeof(EXTRACTOR_KeywordList));
-  result->next = next;    
+  result->next = next;
   result->keyword = keyword;
   result->keywordType = type;
   return result;
@@ -67,7 +66,7 @@ typedef struct {
 
 /* see also: "man 5 deb-control" */
 static Matches tmap[] = {
-  { "Package: ",        EXTRACTOR_SOFTWARE }, 
+  { "Package: ",        EXTRACTOR_SOFTWARE },
   { "Version: ",        EXTRACTOR_VERSIONNUMBER },
   { "Section: ",        EXTRACTOR_GENRE },
   { "Priority: ",       EXTRACTOR_PRIORITY },
@@ -75,7 +74,7 @@ static Matches tmap[] = {
   { "Depends: ",        EXTRACTOR_DEPENDENCY },
   { "Recommends: ",     EXTRACTOR_RELATION },
   { "Suggests: ",       EXTRACTOR_RELATION },
-  { "Installed-Size: ", EXTRACTOR_SIZE }, 
+  { "Installed-Size: ", EXTRACTOR_SIZE },
   { "Maintainer: ",     EXTRACTOR_PACKAGER },
   { "Description: ",    EXTRACTOR_DESCRIPTION },
   { "Source: ",         EXTRACTOR_SOURCE },
@@ -115,10 +114,10 @@ static struct EXTRACTOR_Keywords * processControl(const char * data,
       colon++;
     eol = colon;
     while ( (eol < size) &&
-	    (data[eol] != '\n') ||
-	    ( (eol+1 < size) &&
-	      (data[eol+1] == ' ') ) )
-      eol++;    
+	    ( (data[eol] != '\n') ||
+	      ( (eol+1 < size) &&
+	        (data[eol+1] == ' ') ) ) )
+      eol++;
     if ( (eol == colon) || (eol > size) )
       return prev;
     key = stndup(&data[pos], colon-pos);
@@ -168,7 +167,7 @@ typedef struct {
 /**
  * Process the control.tar file.
  */
-static struct EXTRACTOR_Keywords * 
+static struct EXTRACTOR_Keywords *
 processControlTar(const char * data,
 		  const size_t size,
 		  struct EXTRACTOR_Keywords * prev) {
@@ -186,23 +185,23 @@ processControlTar(const char * data,
       ustar = (USTarHeader*) &data[pos];
       if (0 == strncmp("ustar",
 		       &ustar->magic[0],
-		       strlen("ustar"))) 
+		       strlen("ustar")))
 	pos += 512; /* sizeof(USTarHeader); */
       else
 	pos += 257; /* sizeof(TarHeader); minus gcc alignment... */
     } else {
       pos += 257; /* sizeof(TarHeader); minus gcc alignment... */
     }
-    
+
     memcpy(buf, &tar->filesize[0], 12);
-    buf[12] = '\0';   
+    buf[12] = '\0';
     if (1 != sscanf(buf, "%12llo", &fsize)) /* octal! Yuck yuck! */
       return prev;
     if ( (pos + fsize > size) ||
 	 (fsize > size) ||
 	 (pos + fsize < pos) )
       return prev;
-    
+
     if (0 == strncmp(&tar->name[0],
 		     "./control",
 		     strlen("./control"))) {
@@ -211,7 +210,7 @@ processControlTar(const char * data,
 			    prev);
     }
     if ( (fsize & 511) != 0)
-      fsize = (fsize |= 511)+1; /* round up! */
+      fsize = (fsize | 511)+1; /* round up! */
     if (pos + fsize < pos)
       return prev;
     pos += fsize;
@@ -219,94 +218,64 @@ processControlTar(const char * data,
   return prev;
 }
 
-
 #define MAX_CONTROL_SIZE (1024 * 1024)
 
-typedef struct {
-  int fd;
-  const char * buf;
-  size_t size;
-} WTC;
+static voidpf Emalloc(voidpf opaque, uInt items, uInt size) {
+  return malloc(size * items);
+}
 
-void * writeThread(void * arg) {
-  size_t pos;
-  int ret;
-  WTC * wtc = (WTC*) arg;
-
-  pos = 0;
-  while (pos < wtc->size) {
-    ret = write(wtc->fd,
-		&wtc->buf[pos],
-		wtc->size - pos);
-    if (ret == -1) {
-      close(wtc->fd);
-      return strerror(errno);
-    }
-    pos += ret;
-  }
-  close(wtc->fd);
-  return NULL;
+static void Efree(voidpf opaque, voidpf ptr) {
+  free(ptr);
 }
 
 /**
  * Process the control.tar.gz file.
  */
-static struct EXTRACTOR_Keywords * 
+static struct EXTRACTOR_Keywords *
 processControlTGZ(const unsigned char * data,
 		  size_t size,
 		  struct EXTRACTOR_Keywords * prev) {
   size_t bufSize;
-  gzFile gzf;
-  int fdes[2];
   char * buf;
-  int flags;
-  size_t ret;
-  WTC wtc;
-  pthread_t pt;
-  void * error;
+  z_stream strm;
 
   bufSize = data[size-4] + 256 * data[size-3] + 65536 * data[size-2] + 256*65536 * data[size-1];
   if (bufSize > MAX_CONTROL_SIZE)
     return prev;
 
-  if (0 != PIPE(fdes))
-    return prev;
-  wtc.fd = fdes[1];
-  wtc.size = size;
-  wtc.buf = data;
-  if (0 != pthread_create(&pt, NULL, &writeThread, &wtc)) {
-    close(fdes[0]);
-    close(fdes[1]);
-    return prev;
-  }
-  gzf = gzdopen(fdes[0], "rb");
-  if (gzf == NULL) {
-    close(fdes[0]);
-    close(fdes[1]);
-    pthread_join(pt, &error);
-    return prev;
-  }
-  buf = malloc(bufSize);  
-  if (buf == NULL) {
-    gzclose(gzf);
-    close(fdes[1]);
-    pthread_join(pt, &error);
-    return prev;
-  }
-  if (bufSize != gzread(gzf, buf, bufSize)) {
+  memset(&strm,
+	 0,
+	 sizeof(z_stream));
+
+  strm.next_in = (Bytef*) data;
+  strm.avail_in = size;
+  strm.total_in = 0;
+  strm.zalloc = &Emalloc;
+  strm.zfree = &Efree;
+  strm.opaque = NULL;
+
+  if (Z_OK == inflateInit2(&strm,
+			   15 + 32)) {
+    buf = malloc(bufSize);
+    if (buf == NULL) {
+      inflateEnd(&strm);
+      return prev;
+    }
+    strm.next_out = (Bytef*) buf;
+    strm.avail_out = bufSize;
+    inflate(&strm,
+	    Z_FINISH);
+    if (strm.total_out > 0) {
+      prev = processControlTar(buf,
+			       strm.total_out,
+			       prev);
+      inflateEnd(&strm);
+      free(buf);
+      return prev;
+    }
     free(buf);
-    gzclose(gzf);
-    close(fdes[1]);
-    pthread_join(pt, &error);
-    return prev;
+    inflateEnd(&strm);
   }
-  close(fdes[1]);
-  pthread_join(pt, &error);
-  gzclose(gzf);  
-  prev = processControlTar(buf,
-			   bufSize,
-			   prev);
-  free(buf);  
   return prev;
 }
 
@@ -319,8 +288,8 @@ typedef struct {
   char filesize[10];
   char trailer[2];
 } ObjectHeader;
- 
-struct EXTRACTOR_Keywords * 
+
+struct EXTRACTOR_Keywords *
 libextractor_deb_extract(const char * filename,
 			 const char * data,
 			 const size_t size,
@@ -347,7 +316,7 @@ libextractor_deb_extract(const char * filename,
       return prev;
 
     memcpy(buf, &hdr->filesize[0], 10);
-    buf[10] = '\0'; 
+    buf[10] = '\0';
     if (1 != sscanf(buf, "%10llu", &fsize))
       return prev;
     pos += sizeof(ObjectHeader);
@@ -357,8 +326,8 @@ libextractor_deb_extract(const char * filename,
       return prev;
     if (0 == strncmp(&hdr->name[0],
 		     "control.tar.gz",
-		     strlen("control.tar.gz"))) { 
-      prev = processControlTGZ(&data[pos],
+		     strlen("control.tar.gz"))) {
+      prev = processControlTGZ((const unsigned char*) &data[pos],
 			       fsize,
 			       prev);
       done++;
@@ -374,7 +343,7 @@ libextractor_deb_extract(const char * filename,
     pos += fsize;
     if (done == 2)
       break; /* no need to process the rest of the archive */
-  }  
+  }
   return prev;
 }
 
