@@ -29,7 +29,7 @@ static EXTRACTOR_KeywordList * addKeyword(EXTRACTOR_KeywordType type,
   if (keyword == NULL)
     return next;
   result = malloc(sizeof(EXTRACTOR_KeywordList));
-  result->next = next;    
+  result->next = next;
   result->keyword = keyword;
   result->keywordType = type;
   return result;
@@ -50,7 +50,7 @@ static Matches tmap[] = {
   { NULL, 0 },
 };
 
-static struct EXTRACTOR_Keywords * parseZZZ(unsigned char * data,
+static struct EXTRACTOR_Keywords * parseZZZ(const char * data,
 					    size_t pos,
 					    size_t len,
 					    struct EXTRACTOR_Keywords * prev) {
@@ -88,7 +88,7 @@ static struct EXTRACTOR_Keywords * parseZZZ(unsigned char * data,
 		 slen);
 	  prev = addKeyword(tmap[i].type,
 			    value,
-			    prev);	  
+			    prev);	
 	  pos += slen + 1;
 	}
       }
@@ -99,14 +99,14 @@ static struct EXTRACTOR_Keywords * parseZZZ(unsigned char * data,
   return prev;
 }
 
-static unsigned int getIntAt(char * data) {
+static unsigned int getIntAt(const void * data) {
   char p[4];
 
   memcpy(p, data, 4); /* ensure alignment! */
   return *(unsigned int*)&p[0];
 }
 
-static unsigned int getShortAt(char * data) {
+static unsigned int getShortAt(const void * data) {
   char p[2];
 
   memcpy(p, data, 2); /* ensure alignment! */
@@ -114,7 +114,7 @@ static unsigned int getShortAt(char * data) {
 }
 
 struct EXTRACTOR_Keywords * libextractor_dvi_extract(const char * filename,
-						     unsigned char * data,
+						     const unsigned char * data,
 						     size_t size,
 						     struct EXTRACTOR_Keywords * prev) {
   unsigned int klen;
@@ -129,7 +129,7 @@ struct EXTRACTOR_Keywords * libextractor_dvi_extract(const char * filename,
     return prev;
   if ( (data[0] != 247) || (data[1] != 2) )
     return prev; /* cannot be dvi or unsupported version */
-  klen = data[14]; 
+  klen = data[14];
 
   pos = size-1;
   while ( (data[pos] == 223) && (pos > 0) )
@@ -140,10 +140,10 @@ struct EXTRACTOR_Keywords * libextractor_dvi_extract(const char * filename,
   pos -= 4;
   /* assert pos at 'post_post tag' */
   if (data[pos] != 249)
-    return prev; 
+    return prev;
   opos = pos;
   pos = ntohl(getIntAt(&data[opos+1]));
-  if (pos+25 > size) 
+  if (pos+25 > size)
     return prev;
   /* assert pos at 'post' command */
   if (data[pos] != 248)
@@ -168,7 +168,7 @@ struct EXTRACTOR_Keywords * libextractor_dvi_extract(const char * filename,
   }
   /* ok, now we believe it's a dvi... */
   pages = malloc(16);
-  snprintf(pages, 
+  snprintf(pages,
 	   16,
 	   "%u",
 	   pageCount);
@@ -176,7 +176,7 @@ struct EXTRACTOR_Keywords * libextractor_dvi_extract(const char * filename,
   comment[klen] = '\0';
   memcpy(comment,
 	 &data[15],
-	 klen);  
+	 klen);
   prev = addKeyword(EXTRACTOR_MIMETYPE,
 		    strdup("application/x-dvi"),
 		    prev);
@@ -197,7 +197,7 @@ struct EXTRACTOR_Keywords * libextractor_dvi_extract(const char * filename,
     case 239: /* zzz1 */
       len = data[pos+1];
       if (pos + 2 + len < size)
-	prev = parseZZZ(data,
+	prev = parseZZZ((const char*) data,
 			pos+2,
 			len,
 			prev);
@@ -206,7 +206,7 @@ struct EXTRACTOR_Keywords * libextractor_dvi_extract(const char * filename,
     case 240: /* zzz2 */
       len = ntohs(getShortAt(&data[pos+1]));
       if (pos + 3 + len < size)
-	prev = parseZZZ(data,
+	prev = parseZZZ((const char*) data,
 			pos+3,
 			len,
 			prev);
@@ -215,16 +215,16 @@ struct EXTRACTOR_Keywords * libextractor_dvi_extract(const char * filename,
     case 241: /* zzz3, who uses that? */
       len = (ntohs(getShortAt(&data[pos+1]))) + 65536 * data[pos+3];
       if (pos + 4 + len < size)
-	prev = parseZZZ(data,
+	prev = parseZZZ((const char*) data,
 			pos+4,
 			len,
 			prev);
       pos += len+4;
-      break;     
+      break;
     case 242: /* zzz4, hurray! */
       len = ntohl(getIntAt(&data[pos+1]));
       if (pos + 1 + len < size)
-	prev = parseZZZ(data,
+	prev = parseZZZ((const char*) data,
 			pos+5,
 			len,
 			prev);
