@@ -36,13 +36,13 @@
 
 #include "convert.h"
 
-static struct EXTRACTOR_Keywords * addKeyword(EXTRACTOR_KeywordList *oldhead, 
+static struct EXTRACTOR_Keywords * addKeyword(EXTRACTOR_KeywordList *oldhead,
 					      char *phrase,
 					      EXTRACTOR_KeywordType type) {
   EXTRACTOR_KeywordList * keyword;
-  
+
   keyword = (EXTRACTOR_KeywordList*) malloc(sizeof(EXTRACTOR_KeywordList));
-  keyword->next = oldhead;    
+  keyword->next = oldhead;
   keyword->keyword = phrase;
   keyword->keywordType = type;
   return keyword;
@@ -90,7 +90,7 @@ static Matches tmap[] = {
 
 
 /* mimetype = audio/mpeg */
-struct EXTRACTOR_Keywords * 
+struct EXTRACTOR_Keywords *
 libextractor_id3v23_extract(const char * filename,
 			    const unsigned char * data,
 			    const size_t size,
@@ -125,12 +125,12 @@ libextractor_id3v23_extract(const char * filename,
     ehdrSize = ( ( (data[10]) << 24 ) |
 		 ( (data[11]) << 16 ) |
 		 ( (data[12]) << 8 ) |
-		 ( (data[12]) << 0 ) );     
+		 ( (data[12]) << 0 ) );
 
     padding  = ( ( (data[15]) << 24 ) |
 		 ( (data[16]) << 16 ) |
 		 ( (data[17]) << 8 ) |
-		 ( (data[18]) << 0 ) );     
+		 ( (data[18]) << 0 ) );
     pos += 4 + ehdrSize;
     if (padding < tsize)
       tsize -= padding;
@@ -151,7 +151,7 @@ libextractor_id3v23_extract(const char * filename,
 	 (csize > tsize) ||
 	 (csize == 0) )
       break;
-    flags = data[pos+8]<<8 + data[pos+9];
+    flags = (data[pos+8]<<8) + data[pos+9];
     if ( ( (flags & 0x80) > 0) /* compressed, not yet supported */ ||
 	 ( (flags & 0x40) > 0) /* encrypted, not supported */ ) {
       pos += 10 + csize;
@@ -160,7 +160,7 @@ libextractor_id3v23_extract(const char * filename,
     i = 0;
     while (tmap[i].text != NULL) {
       if (0 == strncmp(tmap[i].text,
-		       &data[pos],
+		       (const char*) &data[pos],
 		       4)) {
 	char * word;
 	if ( (flags & 0x20) > 0) {
@@ -168,30 +168,30 @@ libextractor_id3v23_extract(const char * filename,
 	  pos++;
 	  csize--;
 	}
+	csize--;
 	/* this byte describes the encoding
 	   try to convert strings to UTF-8
 	   if it fails, then forget it */
 	switch (data[pos+10]) {
 	case 0x00 :
-	  word = convertToUtf8(&data[pos+11],
+	  word = convertToUtf8((const char*) &data[pos+11],
 			       csize,
 			       "ISO-8859-1");
 	  break;
 	case 0x01 :
-	  word = convertToUtf8(&data[pos+11],
+	  word = convertToUtf8((const char*) &data[pos+11],
 			       csize,
 			       "UCS-2");
 	  break;
 	default:
 	  /* bad encoding byte,
 	     try to convert from iso-8859-1 */
-	  word = convertToUtf8(&data[pos+11],
+	  word = convertToUtf8((const char*) &data[pos+11],
 			       csize,
 			       "ISO-8859-1");
 	  break;
 	}
 	pos++;
-	csize--;
 	if ( (word != NULL) &&
 	     (strlen(word) > 0) ) {
 	  prev = addKeyword(prev,
