@@ -36,13 +36,14 @@
 #include "convert.h"
 
 
-static struct EXTRACTOR_Keywords * addKeyword(EXTRACTOR_KeywordList *oldhead, 
-					      char *phrase,
-					      EXTRACTOR_KeywordType type) {
+static struct EXTRACTOR_Keywords * 
+addKeyword(EXTRACTOR_KeywordList *oldhead,
+	   char *phrase,
+	   EXTRACTOR_KeywordType type) {
   EXTRACTOR_KeywordList * keyword;
-  
+
   keyword = (EXTRACTOR_KeywordList*) malloc(sizeof(EXTRACTOR_KeywordList));
-  keyword->next = oldhead;    
+  keyword->next = oldhead;
   keyword->keyword = phrase;
   keyword->keywordType = type;
   return keyword;
@@ -89,7 +90,7 @@ static Matches tmap[] = {
 
 
 /* mimetype = audio/mpeg */
-struct EXTRACTOR_Keywords * 
+struct EXTRACTOR_Keywords *
 libextractor_id3v24_extract(const char * filename,
 			    const unsigned char * data,
 			    const size_t size,
@@ -143,12 +144,12 @@ libextractor_id3v24_extract(const char * filename,
 	      ( (data[pos+5] & 0x7F) << 14 ) |
 	      ( (data[pos+6] & 0x7F) <<  7 ) |
 	      ( (data[pos+7] & 0x7F) <<  0 ) );
-  
+
     if ( (pos + 10 + csize > tsize) ||
 	 (csize > tsize) ||
 	 (csize == 0) )
       break;
-    flags = data[pos+8]<<8 + data[pos+9];
+    flags = (data[pos+8]<<8) + data[pos+9];
     if ( ( (flags & 0x80) > 0) /* compressed, not yet supported */ ||
 	 ( (flags & 0x40) > 0) /* encrypted, not supported */ ) {
       pos += 10 + csize;
@@ -157,7 +158,7 @@ libextractor_id3v24_extract(const char * filename,
     i = 0;
     while (tmap[i].text != NULL) {
       if (0 == strncmp(tmap[i].text,
-		       &data[pos],
+		       (const char*) &data[pos],
 		       4)) {
 	char * word;
 	if ( (flags & 0x20) > 0) {
@@ -171,22 +172,22 @@ libextractor_id3v24_extract(const char * filename,
 	   if it fails, then forget it */
 	switch (data[pos+10]) {
 	case 0x00 :
-	  word = convertToUtf8(&data[pos+11],
+	  word = convertToUtf8((const char*) &data[pos+11],
 			       csize,
 			       "ISO-8859-1");
 	  break;
 	case 0x01 :
-	  word = convertToUtf8(&data[pos+11],
+	  word = convertToUtf8((const char*) &data[pos+11],
 			       csize,
 			       "UTF-16");
 	  break;
 	case 0x02 :
-	  word = convertToUtf8(&data[pos+11],
+	  word = convertToUtf8((const char*) &data[pos+11],
 			       csize,
 			       "UTF-16BE");
 	  break;
 	case 0x03 :
-	  word = malloc(csize+1);	  
+	  word = malloc(csize+1);	
 	  memcpy(word,
 		 &data[pos+11],
 		 csize);
@@ -195,7 +196,7 @@ libextractor_id3v24_extract(const char * filename,
 	default:
 	  /* bad encoding byte,
 	     try to convert from iso-8859-1 */
-	  word = convertToUtf8(&data[pos+11],
+	  word = convertToUtf8((const char*) &data[pos+11],
 			       csize,
 			       "ISO-8859-1");
 	  break;
