@@ -65,26 +65,38 @@ static void NEXT(size_t * end,
 		 const char * buf,
 		 const size_t size) {
   int quot;
-  
+
   quot = 0;
   while ( (*end < size) &&
-	  ( ((quot & 1) != 0) || 
+	  ( ((quot & 1) != 0) ||
 	    ( (buf[*end] != ' ') ) ) ) {
     if (buf[*end] == '\"')
       quot++;
-    (*end)++; 
+    (*end)++;
   }
   if ((quot & 1) == 1)
     (*end) = size+1;
 }
 
-static struct EXTRACTOR_Keywords * tryParse(const char * buf,
-					    size_t size,
-					    struct EXTRACTOR_Keywords * prev) {
+/**
+ * How many bytes do we actually try to scan? (from the beginning
+ * of the file).
+ */
+#define MAX_READ (16 * 1024)
+
+
+
+struct EXTRACTOR_Keywords * 
+libextractor_man_extract(const char * filename,
+			 const char * buf,
+			 size_t size,
+			 struct EXTRACTOR_Keywords * prev) {
   int pos;
   size_t xsize;
   const size_t xlen = strlen(".TH ");
-  
+
+  if (size > MAX_READ)
+    size = MAX_READ;
   pos = 0;
   if (size < xlen)
     return prev;
@@ -100,32 +112,34 @@ static struct EXTRACTOR_Keywords * tryParse(const char * buf,
 	  (buf[xsize] != '\n') )
     xsize++;
   size = xsize;
-  
+
   if (0 == strncmp(".TH ",
 		   &buf[pos],
 		   xlen)) {
-    int end;
-    
+    size_t end;
+
     pos += xlen;
     end = pos;
-    NEXT(&end, buf, size); if (end > size) return prev;
+    NEXT(&end, buf, size); 
+    if (end > size) 
+      return prev;
     if (end - pos > 0) {
       prev = addKeyword(EXTRACTOR_TITLE,
 			stndup(&buf[pos],
 			       end - pos),
-			prev);      
+			prev);
       pos = end + 1;
     }
     if (pos >= size) return prev;
     end = pos;
-    NEXT(&end, buf, size); if (end > size) return prev;    
+    NEXT(&end, buf, size); if (end > size) return prev;
     if (buf[pos] == '\"')
       pos++;
     if ( (end-pos >= 1) && (end - pos <= 4) ) {
       switch (buf[pos]) {
       case '1':
 	prev = addKeyword(EXTRACTOR_CATEGORY,
-			  strdup(_("Commands")), 
+			  strdup(_("Commands")),
 			  prev);
 	break;
       case '2':
@@ -177,7 +191,7 @@ static struct EXTRACTOR_Keywords * tryParse(const char * buf,
       prev = addKeyword(EXTRACTOR_DATE,
 			stndup(&buf[pos],
 			       end - pos),
-			prev);      
+			prev);
       pos = end + 1;
     }
     end = pos;
@@ -186,7 +200,7 @@ static struct EXTRACTOR_Keywords * tryParse(const char * buf,
       prev = addKeyword(EXTRACTOR_SOURCE,
 			stndup(&buf[pos],
 			       end - pos),
-			prev);      
+			prev);
       pos = end + 1;
     }
     end = pos;
@@ -195,39 +209,12 @@ static struct EXTRACTOR_Keywords * tryParse(const char * buf,
       prev = addKeyword(EXTRACTOR_BOOKTITLE,
 			stndup(&buf[pos],
 			       end - pos),
-			prev);      
+			prev);
       pos = end + 1;
-    } 
+    }
   }
 
   return prev;
 }
 
-/**
- * How many bytes do we actually try to scan? (from the beginning
- * of the file).
- */
-#define MAX_READ 2048
-
-struct EXTRACTOR_Keywords * libextractor_man_extract(const char * filename,
-						     char * data,
-						     size_t size,
-						     struct EXTRACTOR_Keywords * prev) {
-  gzFile gz;
-  char * buf;
-  int len;
-  
-  gz = gzopen(filename, "rb");
-  buf = malloc(MAX_READ);
-  len = gzread(gz, buf, MAX_READ);
-  if (len < 0) {
-    free(buf);
-    gzclose(gz);
-    return prev;
-  }
-  gzclose(gz);
-  prev = tryParse(buf, len, prev);
-  free(buf);
-  return prev;
-}
-
+/* end of manextractor.c */
