@@ -250,7 +250,7 @@ Stream *Stream::makeFilter(char *name, Stream *str, Object *params) {
       obj.free();
     }
     str = new FlateStream(str, pred, columns, colors, bits);
-  } 
+  }
 #if 0
 else if (!strcmp(name, "JBIG2Decode")) {
     if (params->isDict()) {
@@ -260,7 +260,7 @@ else if (!strcmp(name, "JBIG2Decode")) {
     globals.free();
   } else if (!strcmp(name, "JPXDecode")) {
     str = new JPXStream(str);
-  } 
+  }
 #endif
 else {
     error(getPos(), "Unknown filter '%s'", name);
@@ -410,18 +410,41 @@ void ImageStream::skipLine() {
 
 StreamPredictor::StreamPredictor(Stream *strA, int predictorA,
 				 int widthA, int nCompsA, int nBitsA) {
+  int totalBits;
+
   str = strA;
   predictor = predictorA;
   width = widthA;
   nComps = nCompsA;
   nBits = nBitsA;
+  predLine = NULL;
+  ok = gFalse;
 
+  if (width <= 0 || nComps <= 0 || nBits <= 0 ||
+      nComps >= INT_MAX/nBits ||
+      width >= INT_MAX/nComps/nBits) {
+    return;
+  }
   nVals = width * nComps;
+  if (nVals + 7 <= 0) {
+    return;
+  }
+
+  totalBits = nVals * nBits;
+  if (totalBits == 0 ||
+       (totalBits / nBits) / nComps != width ||
+       totalBits + 7 < 0) {
+    return;
+  }
   pixBytes = (nComps * nBits + 7) >> 3;
-  rowBytes = ((nVals * nBits + 7) >> 3) + pixBytes;
+  rowBytes = ((totalBits + 7) >> 3) + pixBytes;
+  if (rowBytes < 0) {
+   return;
+  }
   predLine = (Guchar *)gmalloc(rowBytes);
   memset(predLine, 0, rowBytes);
   predIdx = rowBytes;
+  ok = gTrue;
 }
 
 StreamPredictor::~StreamPredictor() {
@@ -1015,6 +1038,10 @@ LZWStream::LZWStream(Stream *strA, int predictor, int columns, int colors,
     FilterStream(strA) {
   if (predictor != 1) {
     pred = new StreamPredictor(this, predictor, columns, colors, bits);
+    if (! pred->isOk()) {
+      delete pred;
+      pred = NULL;
+    }	    
   } else {
     pred = NULL;
   }
@@ -1258,16 +1285,15 @@ GBool RunLengthStream::fillBuf() {
 CCITTFaxStream::CCITTFaxStream(Stream *strA, int encodingA, GBool endOfLineA,
 			       GBool byteAlignA, int columnsA, int rowsA,
 			       GBool endOfBlockA, GBool blackA):
-    FilterStream(strA) {
+  FilterStream(strA) {
   encoding = encodingA;
   endOfLine = endOfLineA;
   byteAlign = byteAlignA;
   columns = columnsA;
+
   rows = rowsA;
   endOfBlock = endOfBlockA;
   black = blackA;
-  refLine = (short *)gmalloc((columns + 3) * sizeof(short));
-  codingLine = (short *)gmalloc((columns + 2) * sizeof(short));
 
   eof = gFalse;
   row = 0;
@@ -1276,8 +1302,14 @@ CCITTFaxStream::CCITTFaxStream(Stream *strA, int encodingA, GBool endOfLineA,
   codingLine[0] = 0;
   codingLine[1] = refLine[2] = columns;
   a0 = 1;
-
   buf = EOF;
+  if (columns + 4 < 1 || (columns + 4) >= INT_MAX / sizeof(short)) {
+    /* illegal value, bail out */
+    eof = gTrue;
+    columns = 0;
+  }
+  refLine = (short *)gmalloc((columns + 3) * sizeof(short));
+  codingLine = (short *)gmalloc((columns + 2) * sizeof(short));  
 }
 
 CCITTFaxStream::~CCITTFaxStream() {
@@ -1532,7 +1564,7 @@ int CCITTFaxStream::lookChar() {
 	eatBits(1);
 	code1 = lookBits(13);
       } while ((code1 >> 1) != 0x001);
-      eatBits(12); 
+      eatBits(12);
       if (encoding > 0) {
 	eatBits(1);
 	nextLine2D = !(code1 & 1);
@@ -2900,6 +2932,10 @@ GBool DCTStream::readBaselineSOF() {
   height = read16();
   width = read16();
   numComps = str->getChar();
+  if (numComps <= 0 || numComps > 4) {
+    return gFalse;
+  }
+
   if (prec != 8) {
     error(getPos(), "Bad DCT precision %d", prec);
     return gFalse;
@@ -2926,6 +2962,9 @@ GBool DCTStream::readProgressiveSOF() {
   height = read16();
   width = read16();
   numComps = str->getChar();
+  if (numComps <= 0 || numComps > 4) {
+    return gFalse;
+  }
   if (prec != 8) {
     error(getPos(), "Bad DCT precision %d", prec);
     return gFalse;
@@ -2948,6 +2987,11 @@ GBool DCTStream::readScanInfo() {
 
   length = read16() - 2;
   scanInfo.numComps = str->getChar();
+  if (scanInfo.numComps <= 0 || scanInfo.numComps > 4) {
+    scanInfo.numComps = 0;
+    error(getPos(), "Bad number of components in DCT stream");
+    return gFalse;
+  }
   --length;
   if (length != 2 * scanInfo.numComps + 3) {
     error(getPos(), "Bad DCT scan info block");
@@ -3022,12 +3066,12 @@ GBool DCTStream::readHuffmanTables() {
   while (length > 0) {
     index = str->getChar();
     --length;
-    if ((index & 0x0f) >= 4) {
+    if (((index & 0x0f) >= 4) || ((index & ~0x10) < 0)) {
       error(getPos(), "Bad DCT Huffman table");
       return gFalse;
     }
     if (index & 0x10) {
-      index &= 0x0f;
+      index &= 0x03;
       if (index >= numACHuffTables)
 	numACHuffTables = index+1;
       tbl = &acHuffTables[index];
@@ -3258,6 +3302,10 @@ FlateStream::FlateStream(Stream *strA, int predictor, int columns,
     FilterStream(strA) {
   if (predictor != 1) {
     pred = new StreamPredictor(this, predictor, columns, colors, bits);
+    if (! pred->isOk()) { 
+      delete pred;
+      pred = NULL;
+    }  
   } else {
     pred = NULL;
   }
