@@ -48,25 +48,25 @@ extern "C" {
 						char * keyword,
 						struct EXTRACTOR_Keywords * next) {
     EXTRACTOR_KeywordList * result;
-    
+
     if (keyword == NULL)
       return next;
     result = (EXTRACTOR_KeywordList*) malloc(sizeof(EXTRACTOR_KeywordList));
-    result->next = next;    
+    result->next = next;
     result->keyword = keyword;
     result->keywordType = type;
     return result;
   }
-  
-  
-  static struct EXTRACTOR_Keywords * printInfoString(Dict *infoDict, 
-						     char *key, 
+
+
+  static struct EXTRACTOR_Keywords * printInfoString(Dict *infoDict,
+						     char *key,
 						     EXTRACTOR_KeywordType type,
 						     struct EXTRACTOR_Keywords * next) {
     Object obj;
     GString *s1;
     char * s;
-    
+
     if (infoDict->lookup(key, &obj)->isString()) {
       s1 = obj.getString();
       s = s1->getCString();
@@ -87,25 +87,53 @@ extern "C" {
 	  con = (char*) convertToUtf8((const char*) u, 2, "UNICODE");
 	  strcat(result, con);
 	  free(con);
-	}		       
+	}
 	next = addKeyword(type,
 			  strdup(result),
 			  next);
 	free(result);
       } else {
-	next = addKeyword(type, 
-			  convertToUtf8(s,
-					strlen(s),
+        unsigned int len = (NULL == s) ? 0 : strlen(s);
+
+        while(0 < len) {
+        /*
+         * Avoid outputting trailing spaces.
+         *
+         * The following expression might be rewritten as
+         * (! isspace(s[len - 1]) && 0xA0 != s[len - 1]).
+         * There seem to exist isspace() implementations 
+         * which do return non-zero from NBSP (maybe locale-dependent).
+         * Remove ISO-8859 non-breaking space (NBSP, hex value 0xA0) from
+         * the expression if it looks suspicious (locale issues for instance).
+         *
+         * Squeezing out all non-printable characters might also be useful.
+         */
+          if ( (' '  != s[len - 1]) && ((char)0xA0 != s[len - 1]) &&
+               ('\r' != s[len - 1]) && ('\n' != s[len - 1]) &&
+               ('\t' != s[len - 1]) && ('\v' != s[len - 1]) &&
+               ('\f' != s[len - 1]) )
+             break;
+
+          else
+            len --;
+        }
+
+        /* there should be a check to truncate preposterously long values. */
+
+        if (0 < len) {
+	  next = addKeyword(type,
+	  		  convertToUtf8(s, len,
 					"ISO-8859-1"),
 			  next);
+        }
       }
     }
     obj.free();
     return next;
   }
-  
-  static struct EXTRACTOR_Keywords * printInfoDate(Dict *infoDict, 
-						   char *key, 
+
+  static struct EXTRACTOR_Keywords * printInfoDate(Dict *infoDict,
+						   char *key,
 						   EXTRACTOR_KeywordType type,
 						   struct EXTRACTOR_Keywords * next) {
     Object obj;
@@ -134,7 +162,7 @@ extern "C" {
 	  con = (char*) convertToUtf8((const char*) u, 2, "UNICODE");
 	  strcat(result, con);
 	  free(con);
-	}		       
+	}		
 	next = addKeyword(type,
 			  strdup(result),
 			  next);
@@ -151,7 +179,7 @@ extern "C" {
     return next;
   }
 
-  
+
   /* which mime-types should not be subjected to
      the PDF extractor? (no use trying!) */
   static char * blacklist[] = {
@@ -178,8 +206,8 @@ extern "C" {
     "video/asf",
     "video/quicktime",
     NULL,
-  };  
-  
+  };
+
 
   static const char *
   extractLast (const EXTRACTOR_KeywordType type,
@@ -195,16 +223,17 @@ extern "C" {
     return result;
   }
 
-  struct EXTRACTOR_Keywords * libextractor_pdf_extract(char * filename,
-                                                       unsigned char * data,
+  struct EXTRACTOR_Keywords * libextractor_pdf_extract(const char * filename,
+						       char * data,
                                                        size_t size,
                                                        struct EXTRACTOR_Keywords * prev) {
     PDFDoc * doc;
-    GString * fileName;
     Object info;
+    Object obj;
+    BaseStream * stream;
     struct EXTRACTOR_Keywords * result;
     const char * mime;
-    
+
     /* if the mime-type of the file is blacklisted, don't
        run the printable extactor! */
     mime = extractLast(EXTRACTOR_MIMETYPE,
@@ -218,45 +247,49 @@ extern "C" {
 
 	j++;
       }
-    }  
+    }
 
-    fileName = new GString(filename);
     /* errorInit();   -- keep commented out, otherwise errors are printed to stderr for non-pdf files! */
-    initParams(".xpdfrc", ".xpdfrc");
-    doc = new PDFDoc(fileName, NULL, NULL);
+    obj.initNull();
+    stream = new MemStream(data, 0, size, &obj);
+    doc = new PDFDoc(stream, NULL, NULL);
     if (! doc->isOk()) {
       delete doc;
-      freeParams();
       return prev;
     }
-    
+
     result = addKeyword(EXTRACTOR_MIMETYPE,
 			strdup("application/pdf"),
 			prev);
     doc->getDocInfo(&info);
     if (info.isDict()) {
-      result = printInfoString(info.getDict(), 
-			       "Title", 
+      result = printInfoString(info.getDict(),
+			       "Title",
 			       EXTRACTOR_TITLE,
 			       result);
-      result = printInfoString(info.getDict(), 
-			       "Subject",      
+      result = printInfoString(info.getDict(),
+			       "Subject",
 			       EXTRACTOR_SUBJECT,
 			       result);
-      result = printInfoString(info.getDict(), 
+      result = printInfoString(info.getDict(),
 			       "Keywords",
 			       EXTRACTOR_KEYWORDS,
 			       result);
-      result = printInfoString(info.getDict(), 
+      result = printInfoString(info.getDict(),
 			       "Author",
 			       EXTRACTOR_AUTHOR,
 			       result);
-      result = printInfoString(info.getDict(), 
+      /*
+       * we now believe that Adobe's Creator
+       * is not a person nor an organisation,
+       * but just a piece of software.
+       */
+      result = printInfoString(info.getDict(),
 			       "Creator",
-			       EXTRACTOR_CREATOR,
+			       EXTRACTOR_SOFTWARE,
 			       result);
-      result = printInfoString(info.getDict(), 
-			       "Producer",     
+      result = printInfoString(info.getDict(),
+			       "Producer",
 			       EXTRACTOR_PRODUCER,
 			       result);
       {
@@ -273,11 +306,11 @@ extern "C" {
 			    strdup(pcnt),
 			    result);
       }
-      result = printInfoDate(info.getDict(),   
-			     "CreationDate", 
+      result = printInfoDate(info.getDict(),
+			     "CreationDate",
 			     EXTRACTOR_CREATION_DATE,
 			     result);
-      result = printInfoDate(info.getDict(),   
+      result = printInfoDate(info.getDict(),
 			     "ModDate",
 			     EXTRACTOR_MODIFICATION_DATE,
 			     result);
@@ -285,8 +318,17 @@ extern "C" {
 
     info.free();
     delete doc;
-    freeParams();
-    
-    return result;  
+
+    return result;
   }
 }
+
+
+
+void __attribute__ ((constructor)) xpdf_init(void) {
+  initParams(".xpdfrc", ".xpdfrc");
+}
+
+void __attribute__ ((destructor)) xpdf_done(void) {
+  freeParams();
+}
