@@ -32,6 +32,20 @@ static char * stndup(const char * str,
   return tmp;
 }
 
+/**
+ * strnlen is GNU specific, let's redo it here to be
+ * POSIX compliant.
+ */
+static size_t stnlen(const char * str,
+		     size_t maxlen) {
+  size_t ret;
+  ret = 0;
+  while ( (ret < maxlen) &&
+	  (str[ret] != '\0') )
+    ret++;
+  return ret;
+}
+
 static struct EXTRACTOR_Keywords * addKeyword(EXTRACTOR_KeywordType type,
 					      char * keyword,
 					      struct EXTRACTOR_Keywords * next) {
@@ -40,13 +54,13 @@ static struct EXTRACTOR_Keywords * addKeyword(EXTRACTOR_KeywordType type,
   if (keyword == NULL)
     return next;
   result = malloc(sizeof(EXTRACTOR_KeywordList));
-  result->next = next;    
+  result->next = next;
   result->keyword = keyword;
   result->keywordType = type;
   return result;
 }
 
-static int getIntAt(const char * pos) {
+static int getIntAt(const void * pos) {
   char p[4];
 
   memcpy(p, pos, 4); /* ensure alignment! */
@@ -58,21 +72,21 @@ static struct {
   char * name;
   EXTRACTOR_KeywordType type;
 } tagmap[] = {
-   { "Author" , EXTRACTOR_AUTHOR}, 
+   { "Author" , EXTRACTOR_AUTHOR},
    { "Description" , EXTRACTOR_DESCRIPTION},
    { "Comment", EXTRACTOR_COMMENT},
-   { "Copyright", EXTRACTOR_COPYRIGHT}, 
+   { "Copyright", EXTRACTOR_COPYRIGHT},
    { "Source", EXTRACTOR_SOURCE},
    { "Creation Time", EXTRACTOR_DATE},
    { "Title", EXTRACTOR_TITLE},
    { "Software", EXTRACTOR_SOFTWARE},
    { "Disclaimer", EXTRACTOR_DISCLAIMER},
    { "Warning", EXTRACTOR_WARNING},
-   { "Signature", EXTRACTOR_RESOURCE_IDENTIFIER}, 
+   { "Signature", EXTRACTOR_RESOURCE_IDENTIFIER},
    { NULL, EXTRACTOR_UNKNOWN},
 };
 
-static struct EXTRACTOR_Keywords * processtEXt(const unsigned char * data,
+static struct EXTRACTOR_Keywords * processtEXt(const char * data,
 					       unsigned int length,
 					       struct EXTRACTOR_Keywords * prev) {
   char * keyword;
@@ -80,10 +94,10 @@ static struct EXTRACTOR_Keywords * processtEXt(const unsigned char * data,
   int i;
 
   data += 4;
-  off = strnlen(data, length) + 1;
+  off = stnlen(data, length) + 1;
   if (off >= length)
     return prev; /* failed to find '\0' */
-  keyword = convertToUtf8(&data[off], 
+  keyword = convertToUtf8(&data[off],
 			  length-off,
 			  "ISO-8859-1");
   i = 0;
@@ -100,7 +114,7 @@ static struct EXTRACTOR_Keywords * processtEXt(const unsigned char * data,
 		    prev);
 }
 
-static struct EXTRACTOR_Keywords * processiTXt(const unsigned char * data,
+static struct EXTRACTOR_Keywords * processiTXt(const char * data,
 					       unsigned int length,
 					       struct EXTRACTOR_Keywords * prev) {
   unsigned int pos;
@@ -111,31 +125,31 @@ static struct EXTRACTOR_Keywords * processiTXt(const unsigned char * data,
   int compressed;
   char * buf;
   uLongf bufLen;
-  int ret;  
+  int ret;
 
-  pos = strnlen(data, length)+1;
+  pos = stnlen(data, length)+1;
   if (pos+3 >= length)
     return prev;
   compressed = data[pos++];
   if (compressed && (data[pos++] != 0))
     return prev; /* bad compression method */
   language = &data[pos];
-  if (strnlen(language, length-pos) > 0)
+  if (stnlen(language, length-pos) > 0)
     prev = addKeyword(EXTRACTOR_LANGUAGE,
 		      stndup(language, length-pos),
 		      prev);
-  pos += strnlen(language, length-pos)+1;
+  pos += stnlen(language, length-pos)+1;
   if (pos+1 >= length)
     return prev;
   translated = &data[pos]; /* already in utf-8! */
-  if (strnlen(translated, length-pos) > 0)
+  if (stnlen(translated, length-pos) > 0)
     prev = addKeyword(EXTRACTOR_TRANSLATED,
 		      stndup(translated, length-pos),
 		      prev);
-  pos += strnlen(translated, length-pos)+1;
+  pos += stnlen(translated, length-pos)+1;
   if (pos >= length)
     return prev;
-  
+
   if (compressed) {
     bufLen = 1024 + 2 * (length - pos);
     while (1) {
@@ -151,9 +165,9 @@ static struct EXTRACTOR_Keywords * processiTXt(const unsigned char * data,
 	/* printf("out of memory"); */
 	return prev; /* out of memory */
       }
-      ret = uncompress(buf,
+      ret = uncompress((Bytef*) buf,
 		       &bufLen,
-		       &data[pos],
+		       (const Bytef*) &data[pos],
 		       length - pos);
       if (ret == Z_OK) {
 	/* printf("zlib ok"); */
@@ -167,10 +181,10 @@ static struct EXTRACTOR_Keywords * processiTXt(const unsigned char * data,
     free(buf);
   } else {
     keyword = stndup(&data[pos], length - pos);
-  }  
+  }
   i = 0;
   while (tagmap[i].name != NULL) {
-    if (0 == strcmp(tagmap[i].name, 
+    if (0 == strcmp(tagmap[i].name,
 		    data))
       return addKeyword(tagmap[i].type,
 			keyword, /* already in utf-8 */
@@ -182,7 +196,7 @@ static struct EXTRACTOR_Keywords * processiTXt(const unsigned char * data,
 		    prev);
 }
 
-static struct EXTRACTOR_Keywords * processIHDR(const unsigned char * data,
+static struct EXTRACTOR_Keywords * processIHDR(const char * data,
 					       unsigned int length,
 					       struct EXTRACTOR_Keywords * prev) {
   char * tmp;
@@ -195,14 +209,14 @@ static struct EXTRACTOR_Keywords * processIHDR(const unsigned char * data,
 	   128,
 	   "%ux%u",
 	   htonl(getIntAt(&data[4])),
-	   htonl(getIntAt(&data[8]))); 
+	   htonl(getIntAt(&data[8])));
   return addKeyword(EXTRACTOR_SIZE,
 		    tmp,
 		    prev);
 }
 
 /* not supported... */
-static struct EXTRACTOR_Keywords * processzTXt(const unsigned char * data,
+static struct EXTRACTOR_Keywords * processzTXt(const char * data,
 					       unsigned int length,
 					       struct EXTRACTOR_Keywords * prev) {
   char * keyword;
@@ -213,7 +227,7 @@ static struct EXTRACTOR_Keywords * processzTXt(const unsigned char * data,
   int ret;
 
   data += 4;
-  off = strnlen(data, length) + 1;
+  off = stnlen(data, length) + 1;
   if (off >= length)
     return prev; /* failed to find '\0' */
   if (data[off] != 0)
@@ -234,9 +248,9 @@ static struct EXTRACTOR_Keywords * processzTXt(const unsigned char * data,
       /* printf("out of memory"); */
       return prev; /* out of memory */
     }
-    ret = uncompress(buf,
+    ret = uncompress((Bytef*) buf,
 		     &bufLen,
-		     &data[off],
+		     (const Bytef*) &data[off],
 		     length - off);
     if (ret == Z_OK) {
       /* printf("zlib ok"); */
@@ -269,11 +283,11 @@ static struct EXTRACTOR_Keywords * processzTXt(const unsigned char * data,
 
 
 struct EXTRACTOR_Keywords * libextractor_png_extract(char * filename,
-                                                     const unsigned char * data,
+                                                     const char * data,
                                                      size_t size,
                                                      struct EXTRACTOR_Keywords * prev) {
-  const unsigned char * pos;
-  const unsigned char * end;
+  const char * pos;
+  const char * end;
   struct EXTRACTOR_Keywords * result;
   unsigned int length;
 
@@ -293,19 +307,19 @@ struct EXTRACTOR_Keywords * libextractor_png_extract(char * filename,
     length = htonl(getIntAt(pos));  pos+=4;
     /* printf("Length: %u, pos %u\n", length, pos - data); */
     if ( (pos+4+length+4 > end) ||
-	 (pos+4+length+4 < pos + 4 + length) ) 
+	 (pos+4+length+4 < pos + 8) )
       break;
-    
+
     if (0 == strncmp(pos, "IHDR", 4))
       result = processIHDR(pos, length, result);
-    if (0 == strncmp(pos, "iTXt", 4)) 
+    if (0 == strncmp(pos, "iTXt", 4))
       result = processiTXt(pos, length, result);
-    if (0 == strncmp(pos, "tEXt", 4)) 
+    if (0 == strncmp(pos, "tEXt", 4))
       result = processtEXt(pos, length, result);
-    if (0 == strncmp(pos, "zTXt", 4)) 
+    if (0 == strncmp(pos, "zTXt", 4))
       result = processzTXt(pos, length, result);
     pos += 4+length+4; /* Chunk type, data, crc */
   }
-  return result;  
+  return result;
 }
 
